﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion
using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [StorableClass]
  public abstract class ZDT : MultiObjectiveTestFunction {


    public override IEnumerable<double[]> OptimalParetoFront(int objectives) {
      return PFStore.get(this.ItemName);
    }

    public override double[,] Bounds(int objectives) {
      return new double[,] { { 0, 1 } };
    }

    public override bool[] Maximization(int objectives) {
      return new bool[objectives];
    }

    public override int MinimumSolutionLength {
      get { return 1; }
    }

    public override int MaximumSolutionLength {
      get { return int.MaxValue; }
    }

    public override int MinimumObjectives {
      get { return 2; }
    }

    public override int MaximumObjectives {
      get { return 2; }
    }

    public override double[] ReferencePoint(int objecitives) {
      return new double[] { 11.0, 11.0 };
    }

    public override double BestKnownHypervolume(int objectives) {
      return -1;
    }

    [StorableConstructor]
    protected ZDT(bool deserializing) : base(deserializing) { }
    protected ZDT(MultiObjectiveTestFunction original, Cloner cloner) : base(original, cloner) {
    }
    protected ZDT() : base() { }

    public override double[] Evaluate(RealVector r, int objectives) {
      if (objectives != 2) throw new ArgumentException("ZDT Problems must always have 2 objectives");
      return Evaluate(r);
    }

    public abstract double[] Evaluate(RealVector r);
  }
}
